@@ -11,7 +11,7 @@ use Carp ();
 use B ();
 #use Devel::Peek;
 
-$JSON::PP::VERSION = '2.21001';
+$JSON::PP::VERSION = '2.24000';
 
 @JSON::PP::EXPORT = qw(encode_json decode_json from_json to_json);
 
@@ -489,7 +489,7 @@ sub allow_bigint {
     sub string_to_json {
         my ($self, $arg) = @_;
 
-        $arg =~ s/([\x22\x5c\n\r\t\f\b])/$esc{$1}/eg;
+        $arg =~ s/([\x22\x5c\n\r\t\f\b])/$esc{$1}/g;
         $arg =~ s/\//\\\//g if ($escape_slash);
         $arg =~ s/([\x00-\x08\x0b\x0e-\x1f])/'\\u00' . unpack('H2', $1)/eg;
 
@@ -636,7 +636,6 @@ BEGIN {
     my $ch;   # 1chracter
     my $len;  # text length (changed according to UTF8 or NON UTF8)
     # INTERNAL
-    my $is_utf8;        # must be with UTF8 flag
     my $depth;          # nest counter
     my $encoding;       # json text encoding
     my $is_valid_utf8;  # temp variable
@@ -675,8 +674,6 @@ BEGIN {
         ($utf8, $relaxed, $loose, $allow_bigint, $allow_barekey, $singlequote)
             = @{$idx}[P_UTF8, P_RELAXED, P_LOOSE .. P_ALLOW_SINGLEQUOTE];
 
-        $is_utf8 = 1 if ( $utf8 or utf8::is_utf8( $text ) );
-
         if ( $utf8 ) {
             utf8::downgrade( $text, 1 ) or Carp::croak("Wide character in subroutine entry");
         }
@@ -741,13 +738,14 @@ BEGIN {
         return object() if($ch eq '{');
         return array()  if($ch eq '[');
         return string() if($ch eq '"' or ($singlequote and $ch eq "'"));
-        return number() if($ch =~ /\d/ or $ch eq '-');
+        return number() if($ch =~ /[0-9]/ or $ch eq '-');
         return word();
     }
 
     sub string {
         my ($i, $s, $t, $u);
         my $utf16;
+        my $is_utf8;
 
         ($is_valid_utf8, $utf8_len) = ('', 0);
 
@@ -801,7 +799,7 @@ BEGIN {
                                 decode_error("surrogate pair expected");
                             }
 
-                            if ((my $hex = hex( $u )) > 255) {
+                            if ( ( my $hex = hex( $u ) ) > 127 ) {
                                 $is_utf8 = 1;
                                 $s .= JSON_PP_decode_unicode($u) || next;
                             }
@@ -819,11 +817,22 @@ BEGIN {
                     }
                 }
                 else{
-                    if ($utf8) {
-                        if( !is_valid_utf8($ch) ) {
-                            $at -= $utf8_len;
-                            decode_error("malformed UTF-8 character in JSON string");
+
+                    if ( ord $ch  > 127 ) {
+                        if ( $utf8 ) {
+                            unless( $ch = is_valid_utf8($ch) ) {
+                                $at -= 1;
+                                decode_error("malformed UTF-8 character in JSON string");
+                            }
+                            else {
+                                $at += $utf8_len - 1;
+                            }
                         }
+                        else {
+                            utf8::encode( $ch );
+                        }
+
+                        $is_utf8 = 1;
                     }
 
                     if (!$loose) {
@@ -960,12 +969,13 @@ BEGIN {
             return $o;
         }
         else {
-            while(defined $ch){
+            while (defined $ch) {
                 $k = ($allow_barekey and $ch ne '"' and $ch ne "'") ? bareKey() : string();
                 white();
 
                 if(!defined $ch or $ch ne ':'){
-                    decode_error("Bad object ; ':' expected");
+                    $at--;
+                    decode_error("':' expected");
                 }
 
                 next_chr();
@@ -1003,7 +1013,8 @@ BEGIN {
 
         }
 
-        decode_error("Bad object ; ,or } expected while parsing object/hash");
+        $at--;
+        decode_error(", or } expected while parsing object/hash");
     }
 
 
@@ -1153,20 +1164,19 @@ BEGIN {
 
 
     sub is_valid_utf8 {
-        unless ( $utf8_len ) {
-            $utf8_len = $_[0] =~ /[\x00-\x7F]/  ? 1
-                      : $_[0] =~ /[\xC2-\xDF]/  ? 2
-                      : $_[0] =~ /[\xE0-\xEF]/  ? 3
-                      : $_[0] =~ /[\xF0-\xF4]/  ? 4
-                      : 0
-                      ;
-        }
 
-        return !($utf8_len = 1) unless ( $utf8_len );
+        $utf8_len = $_[0] =~ /[\x00-\x7F]/  ? 1
+                  : $_[0] =~ /[\xC2-\xDF]/  ? 2
+                  : $_[0] =~ /[\xE0-\xEF]/  ? 3
+                  : $_[0] =~ /[\xF0-\xF4]/  ? 4
+                  : 0
+                  ;
 
-        return 1 if (length ($is_valid_utf8 .= $_[0] ) < $utf8_len); # continued
+        return unless $utf8_len;
 
-        return ( $is_valid_utf8 =~ s/^(?:
+        my $is_valid_utf8 = substr($text, $at - 1, $utf8_len);
+
+        return ( $is_valid_utf8 =~ /^(?:
              [\x00-\x7F]
             |[\xC2-\xDF][\x80-\xBF]
             |[\xE0][\xA0-\xBF][\x80-\xBF]
@@ -1176,8 +1186,7 @@ BEGIN {
             |[\xF0][\x90-\xBF][\x80-\xBF][\x80-\xBF]
             |[\xF1-\xF3][\x80-\xBF][\x80-\xBF][\x80-\xBF]
             |[\xF4][\x80-\x8F][\x80-\xBF][\x80-\xBF]
-        )$//x and !($utf8_len = 0) ); # if valid, make $is_valid_utf8 empty and rest $utf8_len.
-
+        )$/x )  ? $is_valid_utf8 : '';
     }
 
 
@@ -1213,8 +1222,12 @@ BEGIN {
         }
 
         Carp::croak (
-            $no_rep ? "$error" : "$error, at character offset $at [\"$mess\"]"
+            $no_rep ? "$error" : "$error, at character offset $at (before \"$mess\")"
         );
+
+#        Carp::croak (
+#            $no_rep ? "$error" : "$error, at character offset $at [\"$mess\"]"
+#        );
     }
 
 
@@ -1245,7 +1258,6 @@ BEGIN {
             at      => $at,
             ch      => $ch,
             len     => $len,
-            is_utf8 => $is_utf8,
             depth   => $depth,
             encoding      => $encoding,
             is_valid_utf8 => $is_valid_utf8,
@@ -1257,12 +1269,16 @@ BEGIN {
 
 sub _decode_surrogates { # from perlunicode
     my $uni = 0x10000 + (hex($_[0]) - 0xD800) * 0x400 + (hex($_[1]) - 0xDC00);
-    return pack('U*', $uni);
+    my $un  = pack('U*', $uni);
+    utf8::encode( $un );
+    return $un;
 }
 
 
 sub _decode_unicode {
-    return pack("U", hex shift);
+    my $un = pack('U', hex shift);
+    utf8::encode( $un );
+    return $un;
 }
 
 
@@ -1324,7 +1340,7 @@ use constant INCR_M_STR  => 1; # inside string
 use constant INCR_M_BS   => 2; # inside backslash
 use constant INCR_M_JSON => 3; # outside anything, count nesting
 
-$JSON::PP::IncrParser::VERSION = '1.00';
+$JSON::PP::IncrParser::VERSION = '1.01';
 
 my $unpack_format = $] < 5.006 ? 'C*' : 'U*';
 
@@ -1332,10 +1348,10 @@ sub new {
     my ( $class ) = @_;
 
     bless {
+        incr_nest    => 0,
         incr_text    => undef,
         incr_parsing => 0,
         incr_p       => 0,
-        
     }, $class;
 }
 
@@ -1381,8 +1397,8 @@ sub incr_parse {
         else { # in scalar context
             $self->{incr_parsing} = 1;
             my $obj = $self->_incr_parse( $coder, $self->{incr_text} );
-            $self->{incr_parsing} = 0;
-            return $obj;
+            $self->{incr_parsing} = 0 if defined $obj; # pointed by Martin J. Evans
+            return $obj ? $obj : undef; # $obj is an empty string, parsing was completed.
         }
 
     }
@@ -1407,52 +1423,52 @@ sub _incr_parse {
        }
     }
 
-        while ( $len > $p ) {
-            my $s = substr( $text, $p++, 1 );
+    while ( $len > $p ) {
+        my $s = substr( $text, $p++, 1 );
 
-            if ( $s eq '"' ) {
-                if ( $self->{incr_mode} != INCR_M_STR  ) {
-                    $self->{incr_mode} = INCR_M_STR;
-                }
-                else {
-                    $self->{incr_mode} = INCR_M_JSON;
-                    unless ( $self->{incr_nest} ) {
-                        last;
-                    }
+        if ( $s eq '"' ) {
+            if ( $self->{incr_mode} != INCR_M_STR  ) {
+                $self->{incr_mode} = INCR_M_STR;
+            }
+            else {
+                $self->{incr_mode} = INCR_M_JSON;
+                unless ( $self->{incr_nest} ) {
+                    last;
                 }
             }
-
-            if ( $self->{incr_mode} == INCR_M_JSON ) {
-
-                if ( $s eq '[' or $s eq '{' ) {
-                    if ( ++$self->{incr_nest} > $coder->get_max_depth ) {
-                        Carp::croak('json text or perl structure exceeds maximum nesting level (max_depth set too low?)');
-                    }
-                }
-                elsif ( $s eq ']' or $s eq '}' ) {
-                    last unless ( --$self->{incr_nest} );
-                }
-            }
-
         }
 
-        $self->{incr_p} = $p;
+        if ( $self->{incr_mode} == INCR_M_JSON ) {
 
-        return if ( $self->{incr_mode} == INCR_M_JSON and $self->{incr_nest} );
+            if ( $s eq '[' or $s eq '{' ) {
+                if ( ++$self->{incr_nest} > $coder->get_max_depth ) {
+                    Carp::croak('json text or perl structure exceeds maximum nesting level (max_depth set too low?)');
+                }
+            }
+            elsif ( $s eq ']' or $s eq '}' ) {
+                last if ( --$self->{incr_nest} <= 0 );
+            }
+        }
 
-        return unless ( length substr( $self->{incr_text}, 0, $p ) );
+    }
 
-        local $Carp::CarpLevel = 2;
+    $self->{incr_p} = $p;
 
-        $self->{incr_p} = $restore;
-        $self->{incr_c} = $p;
+    return if ( $self->{incr_mode} == INCR_M_JSON and $self->{incr_nest} > 0 );
 
-        my ( $obj, $tail ) = $coder->decode_prefix( substr( $self->{incr_text}, 0, $p ) );
+    return '' unless ( length substr( $self->{incr_text}, 0, $p ) );
 
-        $self->{incr_text} = substr( $self->{incr_text}, $p );
-        $self->{incr_p} = 0;
+    local $Carp::CarpLevel = 2;
 
-        return $obj;
+    $self->{incr_p} = $restore;
+    $self->{incr_c} = $p;
+
+    my ( $obj, $tail ) = $coder->decode_prefix( substr( $self->{incr_text}, 0, $p ) );
+
+    $self->{incr_text} = substr( $self->{incr_text}, $p );
+    $self->{incr_p} = 0;
+
+    return $obj or '';
 }
 
 
@@ -2006,7 +2022,6 @@ Returns
             at      => $at,
             ch      => $ch,
             len     => $len,
-            is_utf8 => $is_utf8,
             depth   => $depth,
             encoding      => $encoding,
             is_valid_utf8 => $is_valid_utf8,
@@ -2108,7 +2123,7 @@ Makamaka Hannyaharamitu, E<lt>makamaka[at]cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2008 by Makamaka Hannyaharamitu
+Copyright 2007-2009 by Makamaka Hannyaharamitu
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
